----------------------------------------------------------------------
-- This file is ment to provide a layer of abstraction to buttons	--
-- FlexBar2 buttons do not use Action IDs, most code in this file	--
-- is aimed to emulate the behaviour of Action IDs.					--
----------------------------------------------------------------------

local Action =  FlexBar2:NewModule("Action", "AceEvent-2.0");
Action.Version = 1;

Action.ItemCache = {};
local ItemCache = Action.ItemCache;
function Action:OnEnable()
    self:BuildItemCache();
    self:RegisterEvent("BAG_UPDATE", "BuildItemCache");
    self:RegisterEvent("UNIT_INVENTORY_CHANGED", function(UnitId) if(UnitId == "player") then Action:BuildItemCache(); end end);
end

function Action:BuildItemCache()
    for ItemLink, _ in pairs(ItemCache) do
        for Info, _ in pairs(ItemCache[ItemLink]) do
            ItemCache[ItemLink][Info] = nil;
        end
    end
    for Bag = 0, 4 do
        for Slot =1, GetContainerNumSlots(Bag) do
            local ItemLink = GetContainerItemLink(Bag, Slot);
            if(ItemLink) then
                ItemLink = "itemlink:" .. ItemLink
                if(type(ItemCache[ItemLink]) ~= "table") then ItemCache[ItemLink] = {}; end
                ItemCache[ItemLink].Bag = Bag;
                ItemCache[ItemLink].BagSlot = Slot;
            end
        end
    end
    for Slot = 0, 23 do
        local ItemLink = GetInventoryItemLink("player", Slot);
        if(ItemLink) then
            ItemLink = "itemlink:" .. ItemLink;
            if(type(ItemCache[ItemLink]) ~= "table") then ItemCache[ItemLink] = {}; end
            ItemCache[ItemLink].InvSlot = Slot;
        end
    end
end

function Action:PickupAction(ActionType, ActionAttribute)
    ClearCursor();
    FlexBar2:Debug("PickupAction:", ActionType);
    if(ActionAttribute) then
        if(ActionType == "spell") then
            PickupSpell(ActionAttribute);
        elseif(ActionType == "item") then
            local ItemInfo = ItemCache[ActionAttribute];
            FlexBar2:Debug("Item: ",ActionAttribute);
            if(type(ItemInfo) == "table") then
                if(ItemInfo.Bag and ItemInfo.BagSlot) then
                    PickupContainerItem(ItemInfo.Bag, ItemInfo.BagSlot);
                elseif(ItemInfo.InvSlot) then
                    PickupInventoryItem(ItemInfo.InvSlot);
                else
                    FlexBar2:Print("Item currently not in the inventory, so can not pickup. Sorry :(");
                end                
            else
                FlexBar2:Print("Item currently not in the inventory, so can not pickup. Sorry :(");
            end
        elseif(ActionType == "macro") then
            PickupMacro(ActionAttribute);
        end
    end
end

Action.ButtonMixin = FlexBar2.OO.Mixin
{
    "LockAction",
    "UnlockAction",
    "Disable",
    "Enable",
	"ClearAction",
    "PickupCurrentAction",
	"PlaceAction",
	"GetActionTexture",
    "GetActionCooldown",
	"UpdateActionTexture",
    "UpdateCooldownSpinner",
    "PreClick_DisableButton",
    "PostClick_PickOldAction",
};

function Action.ButtonMixin:Activate()
    self.Frame:RegisterForDrag("LeftButton");
	self:RegisterEvent("MODIFIER_STATE_CHANGED", "UpdateActionTexture");
    self:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN", "UpdateCooldownSpinner");
    self.OldActionCache = {}; 
end

function Action.ButtonMixin:SetDefaults()
    self.InfoTable.LockAction = false;
end

function Action.ButtonMixin:LoadSettings()
    if(self.InfoTable.LockAction == true) then
        self:LockAction();
    else
        self:UnlockAction();
    end
end

function Action.ButtonMixin:Initialize()
    self:UpdateActionTexture();
    self:UpdateCooldownSpinner();
end

function Action.ButtonMixin:LockAction()
    self:UnregisterScript("OnDragStart", "PickupCurrentAction");
    self:UnregisterScript("OnReceiveDrag", "PlaceAction");
    self:UnregisterScript("PreClick", "PreClick_DisableButton");
    self:UnregisterScript("PostClick", "PostClick_PickOldAction");
end

function Action.ButtonMixin:UnlockAction()
    self:RegisterScript("OnDragStart", "PickupCurrentAction");
    self:RegisterScript("OnReceiveDrag", "PlaceAction");
    self:RegisterScript("PreClick", "PreClick_DisableButton");
    self:RegisterScript("PostClick", "PostClick_PickOldAction");
end

function Action.ButtonMixin:Disable()
    if(InCombatLockdown() == nil) then
        self.DisabledActionType = self:GetModifiedAttribute("type");
        self:SetModifiedAttribute("type", nil, false);
    end
end

function Action.ButtonMixin:Enable()
    if(InCombatLockdown() == nil and self.DisabledActionType ~= nil) then
        self:SetModifiedAttribute("type", self.DisabledActionType, false);
        self.DisabledActionType = nil;
    end
end

function Action.ButtonMixin:GetActionTexture()
	local ActionType = self:GetModifiedAttribute("type");
	if(ActionType == "spell") then
		return GetSpellTexture(self:GetModifiedAttribute("spell")) or "";
	elseif(ActionType == "item") then
		local ItemName, ItemLink, ItemRarity, ItemLevel, ItemMinLevel, ItemType, ItemSubType, ItemStackCount, ItemEquipLoc, ItemTexture = GetItemInfo(self:GetModifiedAttribute("item"));
		return ItemTexture or "";
	elseif(ActionType == "macro") then
		local MacroName, MacroTexture, MacroBody, MacroLocal = GetMacroInfo(self:GetModifiedAttribute("macro"));
		return MacroTexture or "";
	else
		return "";
	end
end

function Action.ButtonMixin:UpdateActionTexture()
	self:SetIcon(self:GetActionTexture());
end

function Action.ButtonMixin:UpdateCooldownSpinner() 
    local CooldownFrame = self.Frame.Cooldown;
    local start, duration, enable = self:GetActionCooldown();
    FlexBar2:Debug("Updating cooldown spinner: ", start, duration, enable);
    if(start ~= nil and duration ~= nil and enable ~= nil and start > 0 and duration > 0 and enable > 0) then
        CooldownFrame:SetCooldown(start, duration);
        CooldownFrame:Show();
    else
        CooldownFrame:Hide();
    end
end

function Action.ButtonMixin:GetActionCooldown()
    local ActionType = self:GetModifiedAttribute("type");
    if(ActionType == "spell") then
        return GetSpellCooldown(self:GetModifiedAttribute("spell"));
    elseif(ActionType == "item") then
        if(ItemInfo.Bag and ItemInfo.BagSlot) then
            return GetContainerItemCooldown(ItemInfo.Bag, ItemInfo.BagSlot);
        elseif(ItemInfo.InvSlot) then
            GetContainerItemCooldown(ItemInfo.InvSlot);
        end
    elseif(ActionType == "macro") then
        -- Unsupported as of yet
    end
end

function Action.ButtonMixin:ClearAction()
    if(InCombatLockdown() == nil) then
        self:SetModifiedAttribute("type", nil);
        self:SetModifiedAttribute("spell", nil);
        self:SetModifiedAttribute("macro", nil);
        self:SetModifiedAttribute("item", nil);
        self:UpdateActionTexture();
    end
end

function Action.ButtonMixin:PreClick_DisableButton()
    if(GetCursorInfo() ~= nil) then        
        local OldActionCache = self.OldActionCache;
        OldActionCache.ActionType = self:GetModifiedAttribute("type");
        OldActionCache.ActionAttribute = self:GetModifiedAttribute(OldActionCache.ActionType);
        self:PlaceAction();
        self:Disable();
    end
end

function Action.ButtonMixin:PostClick_PickOldAction()
    local OldActionCache = self.OldActionCache;
    if(OldActionCache.ActionType ~= nil and OldActionCache.ActionType ~= "" and OldActionCache.ActionAttribute ~= nil and OldActionCache.ActionAttribute ~= "") then
        Action:PickupAction(OldActionCache.ActionType, OldActionCache.ActionAttribute);
        OldActionCache.ActionType = nil;
        OldActionCache.ActionAttribute = nil;
    end
    self:Enable();
end

function Action.ButtonMixin:PickupCurrentAction()
	if(InCombatLockdown() == nil) then
        local ActionType = self:GetModifiedAttribute("type");
        Action:PickupAction(ActionType, self:GetModifiedAttribute(ActionType));
        self:ClearAction();
        self:UpdateCooldownSpinner();
        self:UpdateActionTexture();
	end
end

function Action.ButtonMixin:PlaceAction()
    if(InCombatLockdown() == nil) then
        local ActionType, ActionId, ActionBook = GetCursorInfo()
		FlexBar2:Debug("PlaceAction:", ActionType, ActionId, ActionBook);
        if(ActionType ~= nil) then self:PickupCurrentAction(); end
        if(ActionType == "spell") then
            self:SetModifiedAttribute("type", "spell");
            local SpellName, SpellRank = GetSpellName(ActionId, ActionBook);
            if(type(SpellRank) == "string" and string.find(SpellRank, "^Rank %d+$")) then
                SpellName = SpellName .. "(" .. SpellRank .. ")";
            end
            self:SetModifiedAttribute("spell", SpellName);
        elseif(ActionType == "item") then
			self:SetModifiedAttribute("type", "item");
            self:SetModifiedAttribute("item", "itemlink:" .. ActionBook);
        elseif(ActionType == "macro") then
			local MacroName, MacroTexture, MacroBody, MacroLocal = GetMacroInfo(ActionId);
			self:SetModifiedAttribute("type", "macro");
			self:SetModifiedAttribute("macro", ActionId);
        end
        self:UpdateCooldownSpinner();
		self:UpdateActionTexture();
    else
        FlexBar2:Print("Actions cannot be changed in combat, sorry");
    end
end
